﻿using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using Infragistics;
using System.Collections.Generic;
using System.Collections;
using System.Linq;

namespace SimpleGrid
{
	public partial class SimpleGridViewController : UIViewController
	{
		#region Members

		IGGridView gridView;

		customHelper dsh;

		List<cSimpleGrid> lstGridItems;


        #endregion

        #region Constructor

        public SimpleGridViewController () : base ("SimpleGridViewController", null)
		{
		}

		#endregion

		#region View Controller Methods

		public override void DidReceiveMemoryWarning ()
		{
			// Releases the view if it doesn't have a superview.
			base.DidReceiveMemoryWarning ();
			
			// Release any cached data, images, etc that aren't in use.
		}

		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();
			
			//Next, create a new instance of the grid view and place it on the view.
			gridView = new IGGridView (new System.Drawing.RectangleF (0, 0, this.View.Bounds.Width, this.View.Bounds.Height), IGGridViewStyle.IGGridViewStyleDefault);
			gridView.AutoresizingMask = UIViewAutoresizing.All;
		    gridView.HeaderHeight = 0;

			this.View.Add (gridView);

         

		    //Setup the Grid
			SetupGrid ();
		}

		#endregion

		#region Methods

		/// <summary>
		/// This method is responsible for the layour and populate of the grid view
		/// </summary>
		private bool SetupGrid ()
		{
			//First, create a new return variable
			bool bReturn = true;

			try {

				// We're using the basic IGGridViewDataSourceHelper
				// since our data will be displayed in a standard datagrid
				dsh = new customHelper();

				// Since we are manually creating columns, turn off auto-generation
				dsh.AutoGenerateColumns = false;


                customGoalsColumnDefinition type1 = new customGoalsColumnDefinition("PT", this);
                customGoalsColumnDefinition type2 = new customGoalsColumnDefinition("OT", this);
                customGoalsColumnDefinition type3 = new customGoalsColumnDefinition("ST", this);

                dsh.ColumnDefinitions.Add(type1);
                dsh.ColumnDefinitions.Add(type2);
                dsh.ColumnDefinitions.Add(type3);


                //Next, set the data object
                // dsh.Data = lstGridItems.ToArray();


                dsh.AllowEditing = true;

				//Set the DataSource
				gridView.DataSource = dsh;

				//Turn off empty rows
				gridView.EmptyRows = false;

				//Update gridView
				gridView.UpdateData ();

			} catch {
				//Error building the grid, return false.
				bReturn = false;
			}

			//Finally, return the return variable
			return bReturn;
		}

		#endregion

		public class customGoalsColumnDefinition : IGGridViewColumnDefinition
		{
			#region Members

			string _fieldKey;
			SimpleGridViewController mController;

			#endregion

			#region Constructor

			public customGoalsColumnDefinition (string key, SimpleGridViewController controller) : base ()
			{
				_fieldKey = key;
				mController = controller;

			}

			#endregion

			#region Properties

			public override string FieldKey {
				get {
					return _fieldKey;
				}
			}

		    public override bool Editable 
		    {
				get {
					return true;
				}
			}

			#endregion

			#region Override Methods

			public override IGGridViewCell CreateCell (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource)
			{
                IGGridViewCell cell = (IGGridViewCell)gridView.DequeueReusableCell("Cell");

                if (cell == null)
                {
                    cell = new IGGridViewCell("Cell");
                }



                cell.TextLabel.Text = $"{_fieldKey}.{path.ColumnIndex}";

                return cell;
			}

			public override void RemoveEditor (IGGridView gridView, IGCellPath path, IGGridViewDataSourceHelper dataSource, bool cellVisible, bool anotherCellEnteringEditMode)
			{
				gridView.UnregisterKeyboard (!anotherCellEnteringEditMode);

				IGGridViewCell cell = (IGGridViewCell)gridView.ResolveCellForPath (path);

				UITextField field = (UITextField)cell.Subviews [0];

				cSimpleGrid dataRow = (cSimpleGrid)dataSource.ResolveDataObjectForRow (path);

				//Next, determine if this is a new dataRow, this is in the scenario when a blank row has been added. we need to add it to the datasource so it can be saved.
				if (dataRow == null) {
					//Create a new data Row or GoalsInformation object settings is values to empty string.
					dataRow = new cSimpleGrid () {Motion = string.Empty, Left = 0, Right = 0};

					//Get a reference to the data and add the new dataRow
					ArrayList data = new ArrayList (dataSource.Data);
					data.Add (dataRow);

					//Set the data source back which contains the new row.
					dataSource.Data = (NSObject[])data.ToArray(typeof(cSimpleGrid));
				}

				//Set the Value of the dataRow
				dataRow.Motion = cell.TextLabel.Text = field.Text;

				float duration = cellVisible ? dataSource.EditingTransitionDuration : 0.0f;

				UIView.Transition (field, cell.TextLabel, duration, UIViewAnimationOptions.TransitionFlipFromTop, null);

				//Before returning the cell, apply the styling to the cell
				ApplyStylingToCell (cell, path.RowIndex, gridView);

				//THIS IS THE LINE OF CODE THAT I USE WHEN A USER POPULATES THE BLANK ROW, TO AUTO RE-ADD A NEW BLANK ROW AT THE END, SO THERE IS ALWAYS A BLANK ROW.
				gridView.UpdateData ();

			}

			#endregion

		}

		public class customHelper : IGGridViewDataSourceHelper
		{
			public override int NumberOfRowsInSection (IGGridView gridView, int section)
			{
				return 3;
			}


            public override IGGridViewCell CreateCell(IGGridView gridView, IGCellPath path)
            {
                IGCellPath normPath = this.NormalizePath(path);

               // int i = path.RowIndex % 3;

                var colDef = this.ColumnDefinitions.GetItem<NSObject>(path.RowIndex) as IGGridViewColumnDefinition;

                if (colDef != null)
                {
                    IGGridViewCell cell = colDef.CreateCell(gridView, path, this);

                    if (this.AllowEditing && colDef.Editable)
                    {
                        var tapGesture = new UITapGestureRecognizer(this, new MonoTouch.ObjCRuntime.Selector("HandleEdit:"))
                        {
                            NumberOfTapsRequired = 1
                        };

                        cell.RegisterGestures(new NSObject[] { tapGesture });
                    }


                    return cell;
                }

                return null;
            }

            [Export("HandleEdit:")]
            private void HandleEdit(UITapGestureRecognizer gestureRecognizer)
            {

                gestureRecognizer.Reset();

                var cell = gestureRecognizer.View as IGGridViewCell;
                if (cell == null)
                    return;


                var dsh = cell.GridView.DataSource as IGGridViewDataSourceHelper;
                dsh.BeginEditing(cell.Path, cell.GridView);



            }




            public override int NumberOfColumns(IGGridView gridView)
            {
                return 10;
            }

            public override bool ProvideRowHeights(IGGridView gridView)
            {
                return true;
            }

            public override float ResolveRowHeight(IGGridView gridView, IGRowPath path)
            {
                return 40;
            }

            public override IGColumnWidth ResolveColumnWidth(IGGridView gridView, int nint)
            {
                return IGColumnWidth.CreateNumericColumnWidth(180);

            }
            public override int NumberOfSections(IGGridView gridView)
            {
                return 1;
            }



        }

	
	}
}

